///////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2004 Intel Corporation 
// All rights reserved. 
//
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions are met: 
//
// * Redistributions of source code must retain the above copyright notice, 
// this list of conditions and the following disclaimer. 
// * Redistributions in binary form must reproduce the above copyright notice, 
// this list of conditions and the following disclaimer in the documentation 
// and/or other materials provided with the distribution. 
// * Neither name of Intel Corporation nor the names of its contributors 
// may be used to endorse or promote products derived from this software 
// without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL INTEL OR 
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
// OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
///////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <ipp.h>

#include "rsa.h"

char pTextSource[1024] = 
	"Gallia est omnis divisa in partes tres, quarum "
	"unam incolunt Belgae, aliam Aquitani, tertiam "
	"qui ipsorum lingua Celtae, nostra Galli appellantur. "
	"Hi omnis lingua, institutis, legibus inter se differunt.  "
	"Gallos ab Aquitanis Garumna flumen, a Belgis Matrona "
	"et Sequana dividit. ";

void main()
{
	IppStatus st;
	static const int keyLen = 256;
	RSAKeyGen keyGen(keyLen, keyLen/2);
	keyGen.GenerateKeys();

	BigNum plaintext(keyLen/32), cyphertext(keyLen/32),
		decodedtext(keyLen/32);

	char pTextDest[1024];
	int sign, i;
	for (i=0; i<strlen(pTextSource)+1; i+=keyLen/8)
	{
		plaintext.Set(1,(Ipp32u*)&pTextSource[i]);
		st = ippsRSAEncrypt(plaintext.GetBN(),
			cyphertext.GetBN(), keyGen.GetPublicKey());

		st = ippsRSADecrypt(cyphertext.GetBN(),
			decodedtext.GetBN(), keyGen.GetPrivateKey());
		decodedtext.Get(&sign, (Ipp32u*)&pTextDest[i]);
	}
	printf("Decoded text:\n%s\n", pTextDest);

	// The following code prepares the composite integer
	// and public key for distribution
	int nLen, pubKeyLen;
	Ipp32u pN[keyLen/32];
	Ipp32u pKey[keyLen/32];

	ippsRSAKeyGet(pN, &nLen, IppsRSAKeyN,
		keyGen.GetPublicKey());
	ippsRSAKeyGet(pKey, &pubKeyLen, IppsRSAKeyE,
		keyGen.GetPublicKey());

	printf("Compsite integer:\n");
	for (i=0; i<nLen; i++)
		printf("%x ",pN[i]);
	printf("\n");
	printf("Public key:\n");
	for (i=0; i<pubKeyLen; i++)
		printf("%x ",pKey[i]);
	printf("\n");
}
